==> java-util-1.20.0-sources.jar: com/cedarsoftware/util/TrackingMap.java <==
package com.cedarsoftware.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * TrackingMap
 *
 * @author Sean Kellner
 *         <br>
 *         Copyright (c) Cedar Software LLC
 *         <br><br>
 *         Licensed under the Apache License, Version 2.0 (the "License");
 *         you may not use this file except in compliance with the License.
 *         You may obtain a copy of the License at
 *         <br><br>
 *         http://www.apache.org/licenses/LICENSE-2.0
 *         <br><br>
 *         Unless required by applicable law or agreed to in writing, software
 *         distributed under the License is distributed on an "AS IS" BASIS,
 *         WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *         See the License for the specific language governing permissions and
 *         limitations under the License.
 */
public class TrackingMap<K, V> implements Map<K, V> {
    private final Map<K, V> internalMap;
    private final Set<K> readKeys;

    public TrackingMap(Map<K, V> map) {
        if (map == null)
        {
            throw new IllegalArgumentException("Cannot construct a TrackingMap() with null");
        }
        internalMap = map;
        readKeys = new HashSet<>();
    }

    public V get(Object key) {
        V value = internalMap.get(key);
        if (value != null) {
            readKeys.add((K)key);
        }
        return value;
    }

    public V put(K key, V value)
    {
        if (internalMap.containsKey(key))
        {   // Overwrite case - if value is overwritten at same key, count that as a direct map key access.
            readKeys.add(key);
        }
        return internalMap.put(key, value);
    }

    public boolean containsKey(Object key) {
        boolean containsKey = internalMap.containsKey(key);
        if (containsKey) {
            readKeys.add((K)key);
        }
        return containsKey;
    }

    public void putAll(Map<? extends K, ? extends V> m) {
        internalMap.putAll(m);
    }

    public V remove(Object key) {
        readKeys.remove(key);
        return internalMap.remove(key);
    }

    public int size() {
        return internalMap.size();
    }

    public boolean isEmpty() {
        return internalMap.isEmpty();
    }

    public boolean equals(Object other) {
        return other instanceof Map && internalMap.equals(other);
    }

    public int hashCode() {
        return internalMap.hashCode();
    }

    public String toString() {
        return internalMap.toString();
    }

    public void clear() {
        readKeys.clear();
        internalMap.clear();
    }

    public boolean containsValue(Object value) {
        return internalMap.containsValue(value);
    }

    public Collection<V> values() {
        return internalMap.values();
    }

    public Set<K> keySet() {
        return internalMap.keySet();
    }

    public Set<Entry<K, V>> entrySet() {
        return internalMap.entrySet();
    }

    public void expungeUnused() {
        internalMap.keySet().retainAll(readKeys);
    }

    public void informAdditionalUsage(Collection<K> additional) {
        readKeys.addAll(additional);
    }

    public void informAdditionalUsage(TrackingMap<K, V> additional) {
        readKeys.addAll(additional.readKeys);
    }

    public Set<K> keysUsed() { return readKeys; }
}

==> java-util-1.20.2-sources.jar: com/cedarsoftware/util/TrackingMap.java <==
package com.cedarsoftware.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * TrackingMap
 *
 * @author Sean Kellner
 *         <br>
 *         Copyright (c) Cedar Software LLC
 *         <br><br>
 *         Licensed under the Apache License, Version 2.0 (the "License");
 *         you may not use this file except in compliance with the License.
 *         You may obtain a copy of the License at
 *         <br><br>
 *         http://www.apache.org/licenses/LICENSE-2.0
 *         <br><br>
 *         Unless required by applicable law or agreed to in writing, software
 *         distributed under the License is distributed on an "AS IS" BASIS,
 *         WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *         See the License for the specific language governing permissions and
 *         limitations under the License.
 */
public class TrackingMap<K, V> implements Map<K, V> {
    private final Map<K, V> internalMap;
    private final Set<K> readKeys;

    public TrackingMap(Map<K, V> map) {
        if (map == null)
        {
            throw new IllegalArgumentException("Cannot construct a TrackingMap() with null");
        }
        internalMap = map;
        readKeys = new HashSet<>();
    }

    public V get(Object key) {
        V value = internalMap.get(key);
        if (value == null)
        {
            if (containsKey(key))
            {
                readKeys.add((K)key);
            }
        }
        else
        {
            readKeys.add((K) key);
        }
        return value;
    }

    public V put(K key, V value)
    {
        return internalMap.put(key, value);
    }

    public boolean containsKey(Object key) {
        boolean containsKey = internalMap.containsKey(key);
        if (containsKey) {
            readKeys.add((K)key);
        }
        return containsKey;
    }

    public void putAll(Map<? extends K, ? extends V> m) {
        internalMap.putAll(m);
    }

    public V remove(Object key) {
        readKeys.remove(key);
        return internalMap.remove(key);
    }

    public int size() {
        return internalMap.size();
    }

    public boolean isEmpty() {
        return internalMap.isEmpty();
    }

    public boolean equals(Object other) {
        return other instanceof Map && internalMap.equals(other);
    }

    public int hashCode() {
        return internalMap.hashCode();
    }

    public String toString() {
        return internalMap.toString();
    }

    public void clear() {
        readKeys.clear();
        internalMap.clear();
    }

    public boolean containsValue(Object value) {
        return internalMap.containsValue(value);
    }

    public Collection<V> values() {
        return internalMap.values();
    }

    public Set<K> keySet() {
        return internalMap.keySet();
    }

    public Set<Entry<K, V>> entrySet() {
        return internalMap.entrySet();
    }

    public void expungeUnused() {
        internalMap.keySet().retainAll(readKeys);
    }

    public void informAdditionalUsage(Collection<K> additional) {
        readKeys.addAll(additional);
    }

    public void informAdditionalUsage(TrackingMap<K, V> additional) {
        readKeys.addAll(additional.readKeys);
    }

    public Set<K> keysUsed() { return readKeys; }

    public Map getWrappedMap() { return internalMap; }
}
